package org.jbpm.graph;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.jbpm.graph.model.Graph;

public class GraphEditor extends EditorPart {

	public void doSave(IProgressMonitor monitor) {
	}

	public void doSaveAs() {
	}

	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
	}

	public boolean isDirty() {
		return false;
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	public void createPartControl(Composite parent) {
		// Add a viewer to the editor in order to view the embedded editparts.
		// Set the root editpart of the viewer.
		ScrollingGraphicalViewer viewer = new ScrollingGraphicalViewer();
		viewer.setRootEditPart(new ScalableFreeformRootEditPart());
		viewer.createControl(parent);
		viewer.getControl().setBackground(ColorConstants.white);
		// Setup a simple editpart factory. Editparts are associated with a model 
		// object. A different Editpart can be created depending upon the model
		// requesting the creation. This factory creates default implementations of 
		// a AbstractGraphicalEditPart. 
		viewer.setEditPartFactory(new EditPartFactory() {
			public EditPart createEditPart(EditPart context, Object model) {
				// The role of an editpart is twofold :
				// - creating graphical feedback of the associated model in the viewer;
				//   this role is performed in the createFigure method.
				// - playing the role of the controller component as defined in the 
				//   classical MVC triad; it performs this role by installing a number
				//   of so-called edit policies in the createEditPolicies method.
				return new AbstractGraphicalEditPart() {
					// Create a minimal figure with a label to provide some feedback.
					protected IFigure createFigure() {
						IFigure result = new Figure();
						Label label = new Label("Hello!");
						result.add(label);
						return result;
					}
					// No edit policies will be installed at this point. Thus the editpart
					// will not be able to react to user events.
					protected void createEditPolicies() {
					}
				};
			}			
		});
		// Set a Graph instance as the contents of the viewer.
		// This triggers a creation request for the editpart factory.
		// The default editpart will be created and the feedback label will 
		// appear on the canvas.
		viewer.setContents(new Graph());
	}

	public void setFocus() {
	}

}
