package org.jbpm.graph;

import java.util.EventObject;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.RedoAction;
import org.eclipse.gef.ui.actions.UndoAction;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.jbpm.graph.part.GraphicalEditPartFactory;

public class GraphEditor extends EditorPart {
	
	private EditDomain editDomain;
	private ActionRegistry actionRegistry;
	
	public void doSave(IProgressMonitor monitor) {
	}

	public void doSaveAs() {
	}

	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
		init();
	}

	private void init() {
		initEditDomain();
		initActionRegistry();
		initCommandStackListener();
	}
	
	private void initEditDomain() {
		editDomain = new DefaultEditDomain(this);
	}
	
	private void initActionRegistry() {
		actionRegistry = new ActionRegistry();
		actionRegistry.registerAction(new UndoAction(this));
		actionRegistry.registerAction(new RedoAction(this));
	}
	
	private void initCommandStackListener() {
		editDomain.getCommandStack().addCommandStackListener(new CommandStackListener() {
			public void commandStackChanged(EventObject event) {
				Iterator<Action> iterator = actionRegistry.getActions();
				while (iterator.hasNext()) {
					Object action = iterator.next();
					if (action instanceof UpdateAction) {
						((UpdateAction)action).update();
					}
				}
			}			
		});
	}

	public boolean isDirty() {
		return editDomain.getCommandStack().isDirty();
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	public void createPartControl(Composite parent) {
		ScrollingGraphicalViewer viewer = new ScrollingGraphicalViewer();
		viewer.setRootEditPart(new ScalableFreeformRootEditPart());
		viewer.createControl(parent);
		viewer.getControl().setBackground(ColorConstants.white);
		viewer.setEditPartFactory(new GraphicalEditPartFactory());
		viewer.setContents(ContentProvider.INSTANCE.newSampleGraph());
		editDomain.addViewer(viewer);
	}

	public void setFocus() {
	}
	
	public ActionRegistry getActionRegistry() {
		return actionRegistry;
	}
	
	public Object getAdapter(Class adapter) {
		if (adapter == CommandStack.class) {
			return editDomain.getCommandStack();
		}
		return super.getAdapter(adapter);
	}
	
}
