package org.jbpm.graph.command;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.gef.commands.Command;
import org.jbpm.graph.model.Connection;
import org.jbpm.graph.model.Graph;
import org.jbpm.graph.model.Node;

public class NodeDeleteCommand extends Command {
	
	private Node node;
	private List<Connection> connections;
	private Map<Connection, Node> connectionSources;
	private Map<Connection, Node> connectionTargets;
	private Graph graph;
	
	public void execute() {
		detachConnections();
		graph.removeNode(node);		
	}
	
	public void undo() {
		graph.addNode(node);
		reattachConnections();
	}
	
	private void detachConnections() {
		connections = new ArrayList<Connection>();
		connectionSources = new HashMap<Connection, Node>();
		connectionTargets = new HashMap<Connection, Node>();
		connections.addAll(node.getSourceConnections());
		connections.addAll(node.getTargetConnections());
		for (int i = 0; i < connections.size(); i++) {
			Connection connection = (Connection)connections.get(i);
			connectionSources.put(connection, connection.getSource());
			connectionTargets.put(connection, connection.getTarget());
			connection.setSource(null);
			connection.setTarget(null);
		}
	}
	
	private void reattachConnections() {
		for (int i = 0; i < connections.size(); i++) {
			Connection connection = (Connection)connections.get(i);
			connection.setSource((Node)connectionSources.get(connection));
			connection.setTarget((Node)connectionTargets.get(connection));
		}
	}
	
	public void setNode(Node node) {
		this.node = node;
	}

	public void setGraph(Graph graph) {
		this.graph = graph;
	}
}
