package org.jbpm.graph.part;

import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.jbpm.graph.figure.NodeFigure;
import org.jbpm.graph.model.Connection;
import org.jbpm.graph.model.Node;
import org.jbpm.graph.policy.NodeComponentEditPolicy;
import org.jbpm.graph.policy.NodeGraphicalNodeEditPolicy;

public class NodeEditPart extends AbstractGraphicalEditPart implements Observer, org.eclipse.gef.NodeEditPart { 
	
	public NodeEditPart(Node node) {
		setModel(node);
	}

	protected IFigure createFigure() {
		return new NodeFigure();
	}

	public void refreshVisuals() {
		NodeFigure figure = (NodeFigure)getFigure();
		Node node = (Node)getModel();
		GraphEditPart parent = (GraphEditPart)getParent();
		figure.getLabel().setText(node.getName());
		Rectangle r = new Rectangle(node.getConstraint());
		parent.setLayoutConstraint(this, figure, r);
	}
	
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new NodeComponentEditPolicy());
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new NodeGraphicalNodeEditPolicy());
	}

	protected List<Connection> getModelSourceConnections() {
		return ((Node)getModel()).getSourceConnections();
	}
	
	protected List<Connection> getModelTargetConnections() {
		return ((Node)getModel()).getTargetConnections();
	}

	public void activate() {
		if (!isActive()) {
			((Node)getModel()).addObserver(this);
			super.activate();
		}
	}
	
	public void deactivate() {
		if (isActive()) {
			((Node)getModel()).deleteObserver(this);
			super.deactivate();
		}
	}
				
	public void update(Observable observable, Object message) {
		refreshVisuals();		
		refreshSourceConnections();
		refreshTargetConnections();
	}

	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return ((NodeFigure)getFigure()).getConnectionAnchor();
	}

	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return ((NodeFigure)getFigure()).getConnectionAnchor();
	}

	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return ((NodeFigure)getFigure()).getConnectionAnchor();
	}

	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return ((NodeFigure)getFigure()).getConnectionAnchor();
	}

}
