package org.jbpm.graph;

import java.util.EventObject;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.RedoAction;
import org.eclipse.gef.ui.actions.UndoAction;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.WorkbenchPart;
import org.jbpm.graph.model.Graph;
import org.jbpm.graph.part.GraphicalEditPartFactory;

public class GraphEditor extends EditorPart {
	
	private EditDomain editDomain;
	private ActionRegistry actionRegistry;
	
	public void doSave(IProgressMonitor monitor) {
	}

	public void doSaveAs() {
	}

	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
		init();
	}
	
	private void init() {
		initEditDomain();
		initActionRegistry();
		initCommandStackListener();
		initSelectionListener();
	}
	
	private void initEditDomain() {
		editDomain = new DefaultEditDomain(this);
	}
	
	private void initActionRegistry() {
		actionRegistry = new ActionRegistry();
		actionRegistry.registerAction(new UndoAction(this));
		actionRegistry.registerAction(new RedoAction(this));
		actionRegistry.registerAction(new DeleteAction((WorkbenchPart)this));
	}
	
	private void initCommandStackListener() {
		editDomain.getCommandStack().addCommandStackListener(new CommandStackListener() {
			public void commandStackChanged(EventObject event) {
				updateActions();
			}
		});
	}
	
	private void initSelectionListener() {
		ISelectionService selectionService = getSite().getWorkbenchWindow().getSelectionService();
		selectionService.addSelectionListener(new ISelectionListener() {
			public void selectionChanged(IWorkbenchPart part, ISelection selection) {
				updateActions();
			}						
		});		
	}
	
	private void updateActions() {
		Iterator iterator = actionRegistry.getActions();
		while (iterator.hasNext()) {
			Object action = iterator.next();
			if (action instanceof UpdateAction) {
				((UpdateAction)action).update();
			}
		}
	}	
	
	public boolean isDirty() {
		return false;
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	public void createPartControl(Composite parent) {
		SashForm form = new SashForm(parent, SWT.HORIZONTAL);
		createPaletteViewer(form);
		createGraphViewer(form);
		form.setWeights(new int[] { 15, 85 });
	}
	
	private void createPaletteViewer(Composite parent)  {
		PaletteViewer viewer = new PaletteViewer();
		viewer.createControl(parent);
		editDomain.setPaletteViewer(viewer);
		editDomain.setPaletteRoot(new GraphPalette());
	}
	
	private void createGraphViewer(Composite parent) {
		ScrollingGraphicalViewer viewer = new ScrollingGraphicalViewer();
		viewer.setRootEditPart(new ScalableFreeformRootEditPart());
		viewer.createControl(parent);
		viewer.getControl().setBackground(ColorConstants.white);
		viewer.setEditPartFactory(new GraphicalEditPartFactory());
		viewer.setContents(new Graph());
		editDomain.addViewer(viewer);
		getSite().setSelectionProvider(viewer);
	}

	public void setFocus() {
	}
	
	public ActionRegistry getActionRegistry() {
		return actionRegistry;
	}
	
	public Object getAdapter(Class adapter) {
		if (adapter == CommandStack.class) {
			return editDomain.getCommandStack();
		}
		return super.getAdapter(adapter);
	}
	
}
